####################3
#  This is a library to support the CAMERRA method
#  v0.01 2/17/2017
#################3

import numpy as np

class SegmentSegment(object):
    """
    This class covers the Segment Segment object
    """

    def __init__(self):
        self.segment_values = ""

    def importSegmentValues(self, filename):
        self.segment_values = np.loadtxt(filename,dtype={'names': ('s1', 's2','value'),
                                                         'formats': ('i', 'i','S100')})
        return self.segment_values

class ResidueResidue(object):
    """
    This class covers the Residue Residue information



    """

    def __init__(self):
        self.residue_values = ""
        self.max_residue_number = ""

    def convertSStoRR(self, segment_residue_map, segment_values):
        segment_map_length = len(segment_residue_map)
        max_residue_number = segment_residue_map[segment_map_length - 1][1]

        # Initalize output array
        array_size = (max_residue_number * max_residue_number, 3)
        output_array = np.zeros(array_size, dtype='S100')

        o = 0
        q = 0
        output_array_itr = 0
        data_length = len(segment_values[:])

        for m in range(segment_map_length):

            segment_start = int(segment_residue_map[m][0])
            segment_finish = int(segment_residue_map[m][1])

            for k in range(o, segment_map_length):

                segment_start_i = int(segment_residue_map[k][0])
                segment_finish_i = int(segment_residue_map[k][1])

                if q < data_length:
                    result = segment_values[q][2]
                z = segment_start_i

                # SEGMENT A
                for j in range(segment_start, segment_finish + 1):

                    # SEGMENT B
                    for i in range(z, segment_finish_i + 1):
                        output_array[output_array_itr][0] = int(j)
                        output_array[output_array_itr][1] = int(i)
                        output_array[output_array_itr][2] = str(result)
                        output_array_itr = output_array_itr + 1

                        # For the first set, there was a duplicate number, 1 to 2, and 2 to 1
                        # This will write the lower half of the triangle except the duplicate
                        if (i <> j):
                            output_array[output_array_itr][0] = int(i)
                            output_array[output_array_itr][1] = int(j)
                            output_array[output_array_itr][2] = str(result)
                            output_array_itr = output_array_itr + 1

                    if (segment_start == segment_start_i):
                        z = z + 1
                q = q + 1
            o = o + 1
        self.residue_values = output_array
        self.max_residue_number = max_residue_number
        return self.residue_values

    def writeFile(self,filename):
        rv = self.residue_values
        max_num = self.max_residue_number
        f = open(filename,'w')
        for i in range(max_num * max_num):
            f.write(str(int(rv[i][0]))+" "+ str(int(rv[i][1]))+" "+str(rv[i][2])+"\n")
        f.close()

class CamerraParameters(object):
    """
    This is a class that handles the parameter.conf file for segment and residue information


    """

    def __init__(self, filename):
        self.filename = filename
        self.parameter_list = self.getParameters()

    def getParameters(self):
        parameter_list = {}

        # Read parameter.conf file, filter out perl % signs and new lines
        with open(self.filename) as f:
            for line in f:
                if not line.startswith('%') and not line.startswith('\n'):
                    key, value = line.split('=')
                    parameter_list[key] = value.strip()
        return parameter_list

    def getSegments(self):
        segment_list = []
        segment_list.append([])
        segment_list.append([])

        segment_parameter = self.parameter_list['X_SEG'].split(",")
        segment_max = self.parameter_list['X_RSIZE']
        i = 0
        y = 1
        segment_num = len(segment_parameter)

        # This loop covers the segment listing in X_SEG parameter
        for segment in segment_parameter:
            segment_list[0].append(int(y))
            segment_list[1].append(int(segment_parameter[i]))

            # This adds the final segment from the end to X_RSIZE (residue size
            if i == (segment_num - 1):
                segment_list[0].append(int(segment_max))
                segment_list[1].append(int(segment_max))
            y = int(segment_parameter[i]) + 1
            i = i + 1
        return segment_list
